CREATE TABLE private_keys (
	key_uuid TEXT NOT NULL PRIMARY KEY COLLATE NOCASE,
	created REAL NOT NULL,
	last_updated REAL NOT NULL,
	label TEXT NOT NULL COLLATE NOCASE,
	description TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	type TEXT NOT NULL COLLATE NOCASE,
	key_size INTEGER NOT NULL,
	thumbprint TEXT NOT NULL COLLATE NOCASE,
	pem TEXT NOT NULL
);
CREATE INDEX private_keys_label_idx ON private_keys(label);
CREATE UNIQUE INDEX private_keys_thumbprint_uidx ON private_keys(thumbprint);

CREATE TABLE signing_requests (
	request_uuid TEXT NOT NULL PRIMARY KEY COLLATE NOCASE,
	created REAL NOT NULL,
	key_thumbprint TEXT NOT NULL COLLATE NOCASE,
	common_name TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	alternate_names TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	organization TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	organizational_unit TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	country TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	email_address TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	locality TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	state TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	pem TEXT NOT NULL
);
CREATE INDEX signing_requests_key_thumbprint_idx ON signing_requests(key_thumbprint);
CREATE INDEX signing_requests_common_name_idx ON signing_requests(common_name);
CREATE INDEX signing_requests_alternate_names_idx ON signing_requests(alternate_names);
CREATE INDEX signing_requests_organization_idx ON signing_requests(organization);
CREATE INDEX signing_requests_organizational_unit_division_idx ON signing_requests(organizational_unit);
CREATE INDEX signing_requests_country_idx ON signing_requests(country);
CREATE INDEX signing_requests_email_address_name_idx ON signing_requests(email_address);
CREATE INDEX signing_requests_locality_idx ON signing_requests(locality);
CREATE INDEX signing_requests_state_idx ON signing_requests(state);

CREATE TABLE certificates (
	thumbprint TEXT NOT NULL PRIMARY KEY COLLATE NOCASE,
	created REAL NOT NULL,
	last_updated REAL NOT NULL,
	key_thumbprint TEXT COLLATE NOCASE,
	issuer_dn TEXT NOT NULL COLLATE NOCASE,
	subject_dn TEXT NOT NULL COLLATE NOCASE,
	issuer_cn TEXT NOT NULL COLLATE NOCASE,
	subject_cn TEXT NOT NULL COLLATE NOCASE,
	description TEXT NOT NULL DEFAULT '' COLLATE NOCASE,
	expires REAL NOT NULL,
	pem TEXT NOT NULL
);
CREATE INDEX certificates_key_thumbprint_idx ON certificates(key_thumbprint);

CREATE TABLE folders (
	folder_uuid TEXT NOT NULL PRIMARY KEY COLLATE NOCASE,
	path TEXT NOT NULL COLLATE NOCASE
);
CREATE UNIQUE INDEX folders_path_uidx ON folders(path);

CREATE TABLE private_key_folders (
	folder_uuid TEXT NOT NULL COLLATE NOCASE REFERENCES folders(folder_uuid) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED,
	key_uuid TEXT NOT NULL COLLATE NOCASE REFERENCES private_keys(key_uuid) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED
);
CREATE UNIQUE INDEX private_key_folders_folder_uuid_key_uuid_uidx ON private_key_folders(folder_uuid, key_uuid);

CREATE TABLE signing_request_folders (
	folder_uuid TEXT NOT NULL COLLATE NOCASE REFERENCES folders(folder_uuid) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED,
	request_uuid TEXT NOT NULL COLLATE NOCASE REFERENCES signing_requests(request_uuid) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED
);
CREATE UNIQUE INDEX signing_request_folders_folder_uuid_request_uuid_uidx ON signing_request_folders(folder_uuid, request_uuid);

CREATE TABLE certificate_folders (
	folder_uuid TEXT NOT NULL COLLATE NOCASE REFERENCES folders(folder_uuid) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED,
	thumbprint TEXT NOT NULL COLLATE NOCASE REFERENCES certificates(thumbprint) ON DELETE CASCADE ON UPDATE CASCADE DEFERRABLE INITIALLY DEFERRED
);
CREATE UNIQUE INDEX certificate_folders_folder_uuid_thumbprint_uidx ON certificate_folders(folder_uuid, thumbprint);